@@ -1,21 +1,21 @@
 
 /*
- * $Id: HttpHeader.c,v 1.63.2.6 2000/02/14 17:08:24 wessels Exp $
+ * $Id: HttpHeader.c,v 1.67.2.2 2001/01/12 00:51:43 wessels Exp $
  *
  * DEBUG: section 55    HTTP Header
  * AUTHOR: Alex Rousskov
  *
- * SQUID Internet Object Cache  http://squid.nlanr.net/Squid/
+ * SQUID Web Proxy Cache          http://www.squid-cache.org/
  * ----------------------------------------------------------
  *
- *  Squid is the result of efforts by numerous individuals from the
- *  Internet community.  Development is led by Duane Wessels of the
- *  National Laboratory for Applied Network Research and funded by the
- *  National Science Foundation.  Squid is Copyrighted (C) 1998 by
- *  the Regents of the University of California.  Please see the
- *  COPYRIGHT file for full details.  Squid incorporates software
- *  developed and/or copyrighted by other sources.  Please see the
- *  CREDITS file for full details.
+ *  Squid is the result of efforts by numerous individuals from
+ *  the Internet community; see the CONTRIBUTORS file for full
+ *  details.   Many organizations have provided support for Squid's
+ *  development; see the SPONSORS file for full details.  Squid is
+ *  Copyrighted (C) 2001 by the Regents of the University of
+ *  California; see the COPYRIGHT file for full details.  Squid
+ *  incorporates software developed and/or copyrighted by other
+ *  sources; see the CREDITS file for full details.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -101,6 +101,7 @@ static const HttpHeaderFieldAttrs HeadersAttrs[] =
     {"Mime-Version", HDR_MIME_VERSION, ftStr},	/* for now */
     {"Pragma", HDR_PRAGMA, ftStr},
     {"Proxy-Authenticate", HDR_PROXY_AUTHENTICATE, ftStr},
+    {"Proxy-Authentication-Info", HDR_PROXY_AUTHENTICATION_INFO, ftStr},
     {"Proxy-Authorization", HDR_PROXY_AUTHORIZATION, ftStr},
     {"Proxy-Connection", HDR_PROXY_CONNECTION, ftStr},
     {"Public", HDR_PUBLIC, ftStr},
@@ -117,6 +118,7 @@ static const HttpHeaderFieldAttrs HeadersAttrs[] =
     {"Via", HDR_VIA, ftStr},	/* for now */
     {"Warning", HDR_WARNING, ftStr},	/* for now */
     {"WWW-Authenticate", HDR_WWW_AUTHENTICATE, ftStr},
+    {"Authentication-Info", HDR_AUTHENTICATION_INFO, ftStr},
     {"X-Cache", HDR_X_CACHE, ftStr},
     {"X-Cache-Lookup", HDR_X_CACHE_LOOKUP, ftStr},
     {"X-Forwarded-For", HDR_X_FORWARDED_FOR, ftStr},
@@ -149,6 +151,8 @@ static http_hdr_type ListHeadersArr[] =
     HDR_VIA,
     /* HDR_WARNING, */
     HDR_WWW_AUTHENTICATE,
+    HDR_AUTHENTICATION_INFO,
+    HDR_PROXY_AUTHENTICATION_INFO,
     /* HDR_EXPECT, HDR_TE, HDR_TRAILER */
     HDR_X_FORWARDED_FOR
 };
@@ -929,6 +933,11 @@ httpHeaderEntryParseCreate(const char *field_start, const char *field_end)
     /* do we have a valid field name within this field? */
     if (!name_len || name_end > field_end)
 	return NULL;
+    if (name_len > 65536) {
+	/* String has a 64K limit */
+	debug(55, 1) ("WARNING: ignoring header name of %d bytes\n", name_len);
+	return NULL;
+    }
     /* now we know we can parse it */
     e = memAllocate(MEM_HTTP_HDR_ENTRY);
     debug(55, 9) ("creating entry %p: near '%s'\n", e, getStringPrefix(field_start, field_end));
@@ -946,6 +955,15 @@ httpHeaderEntryParseCreate(const char *field_start, const char *field_end)
     /* trim field value */
     while (value_start < field_end && xisspace(*value_start))
 	value_start++;
+    if (field_end - value_start > 65536) {
+	/* String has a 64K limit */
+	debug(55, 1) ("WARNING: ignoring '%s' header of %d bytes\n",
+	    strBuf(e->name), field_end - value_start);
+	if (e->id == HDR_OTHER)
+	    stringClean(&e->name);
+	memFree(e, MEM_HTTP_HDR_ENTRY);
+	return NULL;
+    }
     /* set field value */
     stringLimitInit(&e->value, value_start, field_end - value_start);
     Headers[id].stat.seenCount++;
