@@ -1,20 +1,20 @@
 
 /*
- * $Id: comm_select.c,v 1.35.2.6 2000/05/17 23:51:03 wessels Exp $
+ * $Id: comm_select.c,v 1.47.2.1 2001/01/12 00:51:46 wessels Exp $
  *
  * DEBUG: section 5     Socket Functions
  *
- * SQUID Internet Object Cache  http://squid.nlanr.net/Squid/
+ * SQUID Web Proxy Cache          http://www.squid-cache.org/
  * ----------------------------------------------------------
  *
- *  Squid is the result of efforts by numerous individuals from the
- *  Internet community.  Development is led by Duane Wessels of the
- *  National Laboratory for Applied Network Research and funded by the
- *  National Science Foundation.  Squid is Copyrighted (C) 1998 by
- *  the Regents of the University of California.  Please see the
- *  COPYRIGHT file for full details.  Squid incorporates software
- *  developed and/or copyrighted by other sources.  Please see the
- *  CREDITS file for full details.
+ *  Squid is the result of efforts by numerous individuals from
+ *  the Internet community; see the CONTRIBUTORS file for full
+ *  details.   Many organizations have provided support for Squid's
+ *  development; see the SPONSORS file for full details.  Squid is
+ *  Copyrighted (C) 2001 by the Regents of the University of
+ *  California; see the COPYRIGHT file for full details.  Squid
+ *  incorporates software developed and/or copyrighted by other
+ *  sources; see the CREDITS file for full details.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -34,11 +34,7 @@
 
 #include "squid.h"
 
-#if USE_ASYNC_IO
-#define MAX_POLL_TIME 10
-#else
-#define MAX_POLL_TIME 1000
-#endif
+static int MAX_POLL_TIME = 1000;	/* see also comm_quick_poll_required() */
 
 #ifndef        howmany
 #define howmany(x, y)   (((x)+((y)-1))/(y))
@@ -227,7 +223,7 @@ comm_check_incoming_poll_handlers(int nfds, int *fds)
 #if !ALARM_UPDATES_TIME
     getCurrentTime();
 #endif
-    Counter.syscalls.polls++;
+    statCounter.syscalls.polls++;
     if (poll(pfds, npfds, 0) < 1)
 	return incame;
     for (i = 0; i < npfds; i++) {
@@ -276,7 +272,7 @@ comm_poll_icp_incoming(void)
 	incoming_icp_interval = MAX_INCOMING_INTERVAL;
     if (nevents > INCOMING_ICP_MAX)
 	nevents = INCOMING_ICP_MAX;
-    statHistCount(&Counter.comm_icp_incoming, nevents);
+    statHistCount(&statCounter.comm_icp_incoming, nevents);
 }
 
 static void
@@ -303,7 +299,7 @@ comm_poll_http_incoming(void)
 	incoming_http_interval = MAX_INCOMING_INTERVAL;
     if (nevents > INCOMING_HTTP_MAX)
 	nevents = INCOMING_HTTP_MAX;
-    statHistCount(&Counter.comm_http_incoming, nevents);
+    statHistCount(&statCounter.comm_http_incoming, nevents);
 }
 
 /* poll all sockets; call handlers for those that are ready. */
@@ -330,9 +326,8 @@ comm_poll(int msec)
 	getCurrentTime();
 	start = current_dtime;
 #endif
-#if USE_ASYNC_IO
-	aioCheckCallbacks();
-#endif
+	/* Handle any fs callbacks that need doing */
+	storeDirCallback();
 #if DELAY_POOLS
 	FD_ZERO(&slowfds);
 #endif
@@ -382,9 +377,9 @@ comm_poll(int msec)
 	if (msec > MAX_POLL_TIME)
 	    msec = MAX_POLL_TIME;
 	for (;;) {
-	    Counter.syscalls.polls++;
+	    statCounter.syscalls.polls++;
 	    num = poll(pfds, nfds, msec);
-	    Counter.select_loops++;
+	    statCounter.select_loops++;
 	    if (num >= 0)
 		break;
 	    if (ignoreErrno(errno))
@@ -395,7 +390,7 @@ comm_poll(int msec)
 	    /* NOTREACHED */
 	}
 	debug(5, num ? 5 : 8) ("comm_poll: %d FDs ready\n", num);
-	statHistCount(&Counter.select_fds_hist, num);
+	statHistCount(&statCounter.select_fds_hist, num);
 	/* Check timeout handlers ONCE each second. */
 	if (squid_curtime > last_timeout) {
 	    last_timeout = squid_curtime;
@@ -435,7 +430,7 @@ comm_poll(int msec)
 		else {
 		    F->read_handler = NULL;
 		    hdl(fd, F->read_data);
-		    Counter.select_fds++;
+		    statCounter.select_fds++;
 		    if (commCheckICPIncoming)
 			comm_poll_icp_incoming();
 		    if (commCheckDNSIncoming)
@@ -449,7 +444,7 @@ comm_poll(int msec)
 		if ((hdl = F->write_handler)) {
 		    F->write_handler = NULL;
 		    hdl(fd, F->write_data);
-		    Counter.select_fds++;
+		    statCounter.select_fds++;
 		    if (commCheckICPIncoming)
 			comm_poll_icp_incoming();
 		    if (commCheckDNSIncoming)
@@ -496,7 +491,7 @@ comm_poll(int msec)
 	    if ((hdl = F->read_handler)) {
 		F->read_handler = NULL;
 		hdl(fd, F->read_data);
-		Counter.select_fds++;
+		statCounter.select_fds++;
 		if (commCheckICPIncoming)
 		    comm_poll_icp_incoming();
 		if (commCheckDNSIncoming)
@@ -508,7 +503,7 @@ comm_poll(int msec)
 #endif
 #if !ALARM_UPDATES_TIME
 	getCurrentTime();
-	Counter.select_time += (current_dtime - start);
+	statCounter.select_time += (current_dtime - start);
 #endif
 	return COMM_OK;
     }
@@ -549,7 +544,7 @@ comm_check_incoming_select_handlers(int nfds, int *fds)
 #if !ALARM_UPDATES_TIME
     getCurrentTime();
 #endif
-    Counter.syscalls.selects++;
+    statCounter.syscalls.selects++;
     if (select(maxfd, &read_mask, &write_mask, NULL, &zero_tv) < 1)
 	return incame;
     for (i = 0; i < nfds; i++) {
@@ -600,7 +595,7 @@ comm_select_icp_incoming(void)
 	incoming_icp_interval = MAX_INCOMING_INTERVAL;
     if (nevents > INCOMING_ICP_MAX)
 	nevents = INCOMING_ICP_MAX;
-    statHistCount(&Counter.comm_icp_incoming, nevents);
+    statHistCount(&statCounter.comm_icp_incoming, nevents);
 }
 
 static void
@@ -626,7 +621,7 @@ comm_select_http_incoming(void)
 	incoming_http_interval = MAX_INCOMING_INTERVAL;
     if (nevents > INCOMING_HTTP_MAX)
 	nevents = INCOMING_HTTP_MAX;
-    statHistCount(&Counter.comm_http_incoming, nevents);
+    statHistCount(&statCounter.comm_http_incoming, nevents);
 }
 
 #define DEBUG_FDBITS 0
@@ -661,12 +656,11 @@ comm_select(int msec)
 #if !ALARM_UPDATES_TIME
 	getCurrentTime();
 #endif
-#if USE_ASYNC_IO
-	aioCheckCallbacks();
-#endif
 #if DELAY_POOLS
 	FD_ZERO(&slowfds);
 #endif
+	/* Handle any fs callbacks that need doing */
+	storeDirCallback();
 	if (commCheckICPIncoming)
 	    comm_select_icp_incoming();
 	if (commCheckDNSIncoming)
@@ -734,9 +728,9 @@ comm_select(int msec)
 	for (;;) {
 	    poll_time.tv_sec = msec / 1000;
 	    poll_time.tv_usec = (msec % 1000) * 1000;
-	    Counter.syscalls.selects++;
+	    statCounter.syscalls.selects++;
 	    num = select(maxfd, &readfds, &writefds, NULL, &poll_time);
-	    Counter.select_loops++;
+	    statCounter.select_loops++;
 	    if (num >= 0)
 		break;
 	    if (ignoreErrno(errno))
@@ -751,7 +745,7 @@ comm_select(int msec)
 	    continue;
 	debug(5, num ? 5 : 8) ("comm_select: %d FDs ready at %d\n",
 	    num, (int) squid_curtime);
-	statHistCount(&Counter.select_fds_hist, num);
+	statHistCount(&statCounter.select_fds_hist, num);
 	/* Check lifetime and timeout handlers ONCE each second.
 	 * Replaces brain-dead check every time through the loop! */
 	if (squid_curtime > last_timeout) {
@@ -802,7 +796,7 @@ comm_select(int msec)
 		    F->read_handler = NULL;
 		    commUpdateReadBits(fd, NULL);
 		    hdl(fd, F->read_data);
-		    Counter.select_fds++;
+		    statCounter.select_fds++;
 		    if (commCheckICPIncoming)
 			comm_select_icp_incoming();
 		    if (commCheckDNSIncoming)
@@ -846,7 +840,7 @@ comm_select(int msec)
 		    F->write_handler = NULL;
 		    commUpdateWriteBits(fd, NULL);
 		    hdl(fd, F->write_data);
-		    Counter.select_fds++;
+		    statCounter.select_fds++;
 		    if (commCheckICPIncoming)
 			comm_select_icp_incoming();
 		    if (commCheckDNSIncoming)
@@ -870,7 +864,7 @@ comm_select(int msec)
 		F->read_handler = NULL;
 		commUpdateReadBits(fd, NULL);
 		hdl(fd, F->read_data);
-		Counter.select_fds++;
+		statCounter.select_fds++;
 		if (commCheckICPIncoming)
 		    comm_select_icp_incoming();
 		if (commCheckDNSIncoming)
@@ -916,7 +910,7 @@ comm_select_dns_incoming(void)
 	incoming_dns_interval = MAX_INCOMING_INTERVAL;
     if (nevents > INCOMING_DNS_MAX)
 	nevents = INCOMING_DNS_MAX;
-    statHistCount(&Counter.comm_dns_incoming, nevents);
+    statHistCount(&statCounter.comm_dns_incoming, nevents);
 }
 
 void
@@ -964,7 +958,7 @@ examine_select(fd_set * readfds, fd_set * writefds)
 	    FD_SET(fd, &write_x);
 	else
 	    continue;
-	Counter.syscalls.selects++;
+	statCounter.syscalls.selects++;
 	errno = 0;
 	if (!fstat(fd, &sb)) {
 	    debug(5, 5) ("FD %d is valid.\n", fd);
@@ -1030,7 +1024,7 @@ checkTimeouts(void)
 static void
 commIncomingStats(StoreEntry * sentry)
 {
-    StatCounters *f = &Counter;
+    StatCounters *f = &statCounter;
     storeAppendPrintf(sentry, "Current incoming_icp_interval: %d\n",
 	incoming_icp_interval >> INCOMING_FACTOR);
     storeAppendPrintf(sentry, "Current incoming_dns_interval: %d\n",
@@ -1082,3 +1076,10 @@ commUpdateWriteBits(int fd, PF * handler)
 	nwritefds--;
     }
 }
+
+/* Called by async-io or diskd to speed up the polling */
+void
+comm_quick_poll_required(void)
+{
+    MAX_POLL_TIME = 10;
+}
